// Add the forEach method to HTMLCollection.
NodeList.prototype.forEach = HTMLCollection.prototype.forEach = Array.prototype.forEach;

(function ($, Drupal) {
  'use strict';
  Drupal.behaviors.lbPlus = {
    attach: function (context, settings) {
      const links = document.querySelectorAll('.lb_plus-block_category_link');
      if (links.length) {
        links.forEach((link) => {
          link.addEventListener('click', (event) => {
            event.preventDefault();
          });
        });
      }

      once('lb-plus', '.lb-plus-block-selector').forEach((button) => {
        let dragged = null;
        let currentRegion = null;

        document.addEventListener('dragstart', (event) => {
          if (event.target.draggable) {
            event.dataTransfer.effectAllowed = 'copyMove';
            event.dataTransfer.dropEffect = 'copy';
            dragged = event.target;
            document.querySelectorAll('.layout-builder__region').forEach((region) => region.classList.add('droppable'));
          }
        });

        document.addEventListener('dragenter', (event) => {
          const lbregion = event.target.classList.contains('layout-builder__region') ? event.target : event.target.closest('.layout-builder__region');
          if (lbregion && dragged) {
            lbregion.classList.remove('lb_plus__not-allowed');
            if (currentRegion) {
              currentRegion.classList.remove('droppable-hover');
            }
            currentRegion = lbregion;
            currentRegion.classList.add('droppable-hover');
          }
        });

        /**
         * On hover, if the block type is not allowed for that region, hide the cursor and show a red outline around the
         * dragged element.
         */
        document.addEventListener('dragover', (event) => {
          const lbregion = event.target.classList.contains('layout-builder__layout') ? event.target : event.target.closest('.layout-builder__layout');
          if (lbregion && dragged) {
            const blockType = dragged.dataset.block_id;
            const permissions = settings.lb_plus.block_type_permissions;

            const regex = /^layout--.+$/gm;
            lbregion.classList.forEach((className) => {
              if (regex.exec(className)) {
                const template = className.replaceAll('--', '_').replaceAll('-', '_');
                if (permissions && permissions[template] && permissions[template][blockType] == false) {
                  event.dataTransfer.dropEffect = 'move';
                  lbregion.classList.add('lb_plus__not-allowed');
                }
                else {
                  event.dataTransfer.dropEffect = 'copy';
                  lbregion.classList.remove('lb_plus__not-allowed');
                }
              }
            });
          }
          else {
            event.dataTransfer.dropEffect = 'copy';
            if (lbregion) {
              lbregion.classList.remove('lb_plus__not-allowed');
            }
          }
          event.preventDefault();
        });

        document.addEventListener('dragend', (event) => {
          document.querySelectorAll('.layout-builder__region').forEach((region) => {
            region.classList.remove('droppable');
            region.classList.remove('droppable-hover');
          });
        });

        document.addEventListener('drop', (event) => {
          event.preventDefault();

          const lbregion = event.target.classList.contains('layout-builder__region') ? event.target : event.target.closest('.layout-builder__region');
          if (lbregion && dragged) {
            lbregion.classList.remove('lb_plus__not-allowed');
            const target = lbregion.querySelector('.layout-builder__add-block');
            const [type, delta, region] = target.dataset.layoutBuilderHighlightId.split('-');

            let storage_id = null;
            let block_id = null;
            let action = null;

            if (dragged.classList.contains('lb-plus__row')) {
              storage_id = dragged.dataset.storage_id;
              block_id = dragged.dataset.block_id.split(':').pop();
              action = 'add';
            }
            else if (dragged.classList.contains('lb_plus-block_category_link')) {
              const url = new URL(dragged.href);
              if (url) {
                const parts = url.pathname.replace(/^\/+/g, '').split('/');
                storage_id = parts[4];
                block_id = parts[7];
                action = 'add_default';
              }
            }

            var ajaxSettings = {
              url: settings.path.baseUrl + `lb_plus/${action}/block/overrides/${storage_id}/${delta}/${region}/${block_id}`,
            };
            Drupal.ajax(ajaxSettings).execute();

            // Emit an event on drop.
            const dropEvent = new CustomEvent('lb_plus_drop', {
              detail: {
                region: lbregion,
                target: target,
                dragged: dragged
              }
            });
            document.dispatchEvent(dropEvent);

            // Reset the dragged element.
            dragged = null;
          }
        }, false);
      });
    }
  };
})(jQuery, Drupal);
