<?php

namespace Drupal\lb_plus\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Layout\LayoutPluginManagerInterface;
use Drupal\lb_plus\Mixin\LbPlusTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

class LbPlusLayoutsForm  extends ConfigFormBase {
  use LbPlusTrait;

  /**
   * The layout manager.
   *
   * @var \Drupal\Core\Layout\LayoutPluginManagerInterface
   */
  protected $layoutManager;

  /**
   * ChooseSectionController constructor.
   *
   * @param \Drupal\Core\Layout\LayoutPluginManagerInterface $layout_manager
   *   The layout manager.
   */
  public function __construct(LayoutPluginManagerInterface $layout_manager) {
    $this->layoutManager = $layout_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.manager.core.layout')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'lb_plus_layouts_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['lb_plus.layouts'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('lb_plus.settings')->get('block_type_permissions');

    $form['#tree'] = TRUE;
    $form['block_type_permissions'] = ['#type' => 'container'];

    $definitions = $this->layoutManager->getDefinitions();
    $types = $this->getAvailableBlockTypes();
    if (!empty($definitions) && !empty($types)) {
      foreach ($definitions as $definition_key => $definition) {
        if ($definition_key !== 'layout_builder_blank') {
          $form['block_type_permissions'][$definition_key] = [
            '#type' => 'details',
            '#open' => TRUE,
            '#title' => $definition->getLabel(),
          ];

          foreach ($types as $type) {
            $form['block_type_permissions'][$definition_key][$type->id()] = [
              '#type' => 'checkbox',
              '#title' => $type->label(),
              '#default_value' => !empty($config[$definition_key][$type->id()]),
            ];
          }
        }
      }
    }

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValue(['block_type_permissions']);
    $config = \Drupal::service('config.factory')->getEditable('lb_plus.settings');
    $config->set('block_type_permissions', $values)->save();
  }
}
